/*
 * Copyright (c) 2007 by Brian Marick.
 * Inspired by the Java version of the Hierarchical Data format, from the
 * University of Illinois NCSA and The HDF Group (http://www.hdfgroup.org/).
 * This code is intended for use only for training in test-driven design.
 * See LICENSE.txt
 */

package exampler.hdf;

import exampler.hdf.internal.CompoundDatasetFAQ;
import exampler.hdf.internal.NumberDescription;
import exampler.hdf.internal.converters.Converter;
import exampler.hdf.internal.diskAccess.PileOfBytes;

import java.nio.ByteOrder;
import java.util.Vector;

public class CompoundDataset implements CompoundDatasetFAQ {
    private final String filename;
    private final String datasetName;
    private long dimensions;
    private final Vector<Converter> converters = new Vector<Converter>();
    private final Vector<Object> data = new Vector<Object>();
    private PileOfBytes input;

    // I use strings for constants because they're easier to debug with. (1.4)
    static public final String INTEGER_TYPE = "like an integer";
    static public final String FLOAT_TYPE = "like a float";
    public static final ByteOrder DEFAULT_BYTE_ORDER = ByteOrder.BIG_ENDIAN;


    // Datasets can be defined in different ways.

    @SuppressWarnings({"SameParameterValue"}) // Because only called through tests
    public static CompoundDataset createProgrammatically(String filename, String datasetName, int dimensions) {
        return new CompoundDataset(filename, datasetName, dimensions);
    }

    @SuppressWarnings({"SameParameterValue"}) // Because only called through tests.
    public static CompoundDataset defineFromFile(String filename, String datasetName) {
        CompoundDataset retval = new CompoundDataset(filename, datasetName);
        retval.defineFromFile();
        return retval;
    }

    private void defineFromFile() {
        input = PileOfBytes.openFileForReading(filename);

        dimensions = input.tellMemberDimensions();
        for (int i = 0; i < input.tellHowManyMembers(); i++) {
            NumberDescription description = input.describeMember(i);
            Converter newConverter = Converter.of(description);
            converters.add(newConverter);
        }
    }

    private CompoundDataset(String filename, String datasetName, long dimensions) {
        this.dimensions = dimensions;
        this.filename = filename;
        this.datasetName = datasetName;
    }


    private CompoundDataset(String filename, String datasetName) {
        this(filename, datasetName, 0);
    }


    // Filling is separate from definition.

    public void fillFromFile() {
        for (int i = 0; i < converters.size(); i++) {
            Converter converter = converters.get(i);
            data.add(input.createDataFromBytes(i, converter));
        }
        input.close();
        input = null;
    }


    public void fillPreviousMemberWith(Object object) {
        data.add(object);
    }

    public Vector<Object> getData() { return data; }

    // Datasets can also be saved.

    @SuppressWarnings({"EmptyCatchBlock"})
    public void saveToFile() {
        PileOfBytes writer =
                PileOfBytes.openFileForWriting(filename, this);
        for (int i = 0; i < converters.size(); i++) {
            Converter converter = converters.get(i);
            try {
                Object memberData = data.get(i);
                writer.overwriteAsBytes(memberData, i, converter);
            } catch (ArrayIndexOutOfBoundsException e) { } // Missing elements are OK
        }
        writer.close();
    }

    // Answers to frequently asked questions.

    public int tellHowManyMembers() { return converters.size(); }

    public ByteOrder tellByteOrderOf(int member) {
        return converters.get(member).getByteOrder();
    }

    public long tellMemberDimensions() { return dimensions; }

    public String tellTypeOf(int member) {
        return converters.get(member).getElementType();
    }

    public int tellSizeInBytesOf(int member) {
        return converters.get(member).getBytesPerElement();
    }


    // Util

    public void defineMember(NumberDescription d) {
        converters.add(Converter.of(d));
    }

    public void defineNativeMemberFor(Object o) {
        converters.add(Converter.of(o, ByteOrder.nativeOrder()));
    }

    public void defineMemberFor(Object o) {
        converters.add(Converter.of(o));
    }
    

    public void clear() {
        data.clear();
    }

    public CompoundDataset copyWithoutData() {
        CompoundDataset retval = new CompoundDataset(filename, datasetName, dimensions);
        for (Converter converter : converters) {
            retval.defineMember(converter.getDescription());
        }
        return retval;
    }




    // Boring obvious methods.

    @SuppressWarnings({"SimplifiableIfStatement"})
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        CompoundDataset that = (CompoundDataset) o;

        if (dimensions != that.dimensions) return false;
        if (datasetName != null ? !datasetName.equals(that.datasetName) : that.datasetName != null) return false;
        if (filename != null ? !filename.equals(that.filename) : that.filename != null) return false;
        return equalData(that);
    }


    private boolean equalData(CompoundDataset that) {
        if (this.converters.size() != that.converters.size()) return false;
        for (int i = 0; i < this.converters.size(); i++) {
            Converter thisConverter = this.converters.get(i);
            Converter thatConverter = that.converters.get(i);
            if (! thisConverter.getClass().equals(thatConverter.getClass())) return false;
        }

        if (this.data.size() != that.data.size()) return false;
        for (int i = 0; i < this.data.size(); i++) {
            Object thisData = this.data.get(i);
            Object thatData = that.data.get(i);
            Converter judge = this.converters.get(i);

            if (!judge.equalData(thisData, thatData)) return false;
        }
        return true;

    }

    public int hashCode() {
        int result;
        result = (filename != null ? filename.hashCode() : 0);
        result = 31 * result + (datasetName != null ? datasetName.hashCode() : 0);
        result = 31 * result + (int) dimensions;
        result = 31 * result + (converters != null ? converters.hashCode() : 0);
        result = 31 * result + (data != null ? data.hashCode() : 0);
        return result;
    }

    public String toString() {
        StringBuffer retval = new StringBuffer();
        for (int i = 0; i < converters.size(); i++) {
            retval.append(converters.get(i).toString());
            if (i < data.size()) {
                retval.append (" == ");
                retval.append(data.get(i));
            }
            retval.append("/");
        }
        return retval.toString();
    }


}
