/*
 * Copyright (c) 2007 by Brian Marick.
 * Inspired by the Java version of the Hierarchical Data format, from the
 * University of Illinois NCSA and The HDF Group (http://www.hdfgroup.org/).
 * This code is intended for use only for training in test-driven design.
 * See LICENSE.txt
 */

package exampler.hdf;

import exampler.hdf.internal.Like;
import exampler.hdf.testUtilities.TestFileUtils;

import java.nio.ByteOrder;
import java.util.Arrays;
import java.util.Vector;

public class CompoundDatasetTests extends TestFileUtils {
    private static final String DATASET_NAME = "unimportant";
    private static final int FIVE_DIMENSIONS = 5;


    public void test_round_trip() {
        int[] javaIntData = integersUpThrough(5);
        double[] javaDoubleData = doublesUpThrough(5);

        // Here is how you create and write data.
        CompoundDataset originalDataset =
            CompoundDataset.createProgrammatically(tempFileName, DATASET_NAME, FIVE_DIMENSIONS);

            // Integers to native 4-byte integers
        originalDataset.defineNativeMemberFor(javaIntData);
        originalDataset.fillPreviousMemberWith(javaIntData);

            // Doubles to native (8-byte, IEEE) doubles
        originalDataset.defineNativeMemberFor(javaDoubleData);
        originalDataset.fillPreviousMemberWith(javaDoubleData);

            // Doubles to default (8-byte, big-endian, IEEE)
        originalDataset.defineMemberFor(javaDoubleData);
        originalDataset.fillPreviousMemberWith(javaDoubleData);

            // Integers to a peculiar little-endian 3-byte format
        originalDataset.defineMember(Like.integer(3, ByteOrder.LITTLE_ENDIAN));
        originalDataset.fillPreviousMemberWith(javaIntData);

        originalDataset.saveToFile();

        // To read data, first the dataset is defined from the file...
        CompoundDataset newDataset = CompoundDataset.defineFromFile(tempFileName, DATASET_NAME);
                assertNotEquals(originalDataset, newDataset);
                assertEquals(originalDataset.copyWithoutData(), newDataset);

        // ... then the data is pulled in from the file.
        newDataset.fillFromFile();
                assertEquals(originalDataset, newDataset);

        // The normal way to retrieve that data is as a Vector containing
        // all elements.
        Vector extracted = newDataset.getData();
                assertTrue(Arrays.equals(javaIntData,       (int[]) extracted.get(0)));
                assertTrue(Arrays.equals(javaDoubleData, (double[]) extracted.get(1)));
                assertTrue(Arrays.equals(javaDoubleData, (double[]) extracted.get(2)));
                assertTrue(Arrays.equals(javaIntData,       (int[]) extracted.get(3)));
    }

    public void test_a_saved_file_with_no_data_uses_defaults() {
        CompoundDataset originalDataset =
                CompoundDataset.createProgrammatically(tempFileName, DATASET_NAME, FIVE_DIMENSIONS);
        originalDataset.defineMember(Like.integer(4, ByteOrder.BIG_ENDIAN));
        originalDataset.defineMember(Like.floating(8, ByteOrder.BIG_ENDIAN));
        // No data put in dataset.
        originalDataset.saveToFile();

        CompoundDataset newDataset = CompoundDataset.defineFromFile(tempFileName, DATASET_NAME);
        newDataset.fillFromFile();
        Vector extracted = newDataset.getData();
        assertTrue(Arrays.equals(new int[] { 0, 0, 0, 0, 0 } ,
                                 (int[]) extracted.get(0)));
        assertTrue(Arrays.equals(new double[] { 0.0, 0.0, 0.0, 0.0, 0.0 },
                                 (double[]) extracted.get(1)));
    }


    // Tests of minor functions

    public void test_making_native_gets_byte_order_right() {
        // Fortunately, Java default byte order is opposite of endianness of
        // Intel machines, so this is more likely to be a useful test.
        CompoundDataset dataset = CompoundDataset.createProgrammatically(tempFileName, DATASET_NAME, FIVE_DIMENSIONS);
        dataset.defineNativeMemberFor(new int[0]);
        assertEquals(dataset.tellByteOrderOf(0), ByteOrder.nativeOrder());
    }




    public void test_a_dataset_can_have_its_data_cleared() {
        CompoundDataset originallyWithData = CompoundDataset.createProgrammatically(tempFileName, DATASET_NAME, FIVE_DIMENSIONS);
        originallyWithData.defineNativeMemberFor(new int[0]);
        originallyWithData.fillPreviousMemberWith(integersUpThrough(FIVE_DIMENSIONS));

        CompoundDataset withoutData = CompoundDataset.createProgrammatically(tempFileName, DATASET_NAME, FIVE_DIMENSIONS);
        withoutData.defineNativeMemberFor(new int[0]);

        assertNotEquals(withoutData, originallyWithData);

        originallyWithData.clear();
        assertEquals(withoutData, originallyWithData);
    }

    public void test_a_dataset_can_make_a_copy_but_only_of_metadata() {
        CompoundDataset original = CompoundDataset.createProgrammatically(tempFileName, DATASET_NAME, FIVE_DIMENSIONS);
        original.defineNativeMemberFor(new int[0]);
        original.fillPreviousMemberWith(integersUpThrough(FIVE_DIMENSIONS));

        CompoundDataset copy = original.copyWithoutData();
        assertNotEquals(original, copy);

        // But it is equal to one without the data.
        CompoundDataset freshlyEmpty = CompoundDataset.createProgrammatically(tempFileName, DATASET_NAME, FIVE_DIMENSIONS);
        freshlyEmpty.defineNativeMemberFor(new int[4]);

        assertEquals(freshlyEmpty, copy);
    }

    public void test_member_count_and_type_required_for_equality() {
        CompoundDataset one = CompoundDataset.createProgrammatically(tempFileName, DATASET_NAME, FIVE_DIMENSIONS);
        CompoundDataset two = CompoundDataset.createProgrammatically(tempFileName, DATASET_NAME, FIVE_DIMENSIONS);
        assertEquals(one, two);

        one.defineNativeMemberFor(new int[0]);
        assertNotEquals(one, two);

        two.defineNativeMemberFor(new int[0]);
        assertEquals(one, two);

        one.fillPreviousMemberWith(integersUpThrough(FIVE_DIMENSIONS));
        assertNotEquals(one, two);

        two.fillPreviousMemberWith(integersUpThrough(FIVE_DIMENSIONS));
        assertEquals(one, two);

        one.defineNativeMemberFor(new int[0]);
        one.defineNativeMemberFor(new double[0]);
        assertNotEquals(one, two);
    }


}
