/*
 * Copyright (c) 2007 by Brian Marick.
 * Inspired by the Java version of the Hierarchical Data format, from the
 * University of Illinois NCSA and The HDF Group (http://www.hdfgroup.org/).
 * This code is intended for use only for training in test-driven design.
 * See LICENSE.txt
 */

package exampler.hdf.internal.converters;

import junit.framework.TestCase;
import exampler.hdf.internal.Like;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;

public class DoubleArray_8Byte_ConverterTest extends TestCase {


    public void test_can_compare_arrays_of_doubles() {
        Converter converter = Converter.of(new double[3]);
        double[] one = {1.0, 2.0, 3.0 };
        double[] same = {1.0, 2.0, 3.0 };
        double[] diff = {10.0, 2.0, 3.0 };
        assertTrue(converter.equalData(one, same));
        assertFalse(converter.equalData(one, diff));
    }


    // Converting to bytes

    private final double[] doubles = new double[] { 18.0, 22.34 };
    private final byte[] bigEndianBytes =    new byte[]{ 64, 50,  0,  0,  0,   0,         0,         0,
                                                   64, 54, 87, 10, 61, 112, (byte)-93, (byte)-41 };
    private final byte[] littleEndianBytes = new byte[]{         0,         0,   0,  0,  0,  0, 50, 64,
                                                   (byte)-41, (byte)-93, 112, 61, 10, 87, 54, 64};


    private void toBytes(Converter converter, double[] doubles, byte[] expected) {

        byte[] destination = {  5, 5, 5, 5, 5, 5, 5, 5, 4, 4, 4, 4, 4, 4, 4, 4 };
        ByteBuffer buf = ByteBuffer.wrap(destination);
        converter.overwriteAsBytes(doubles, buf);

        assertTrue(Arrays.equals(expected, destination));
    }

    public void test_conversion_of_java_doubles_to_big_endian_bytes() {
        toBytes(Converter.of(doubles, ByteOrder.BIG_ENDIAN),
                doubles,
                bigEndianBytes);
    }

    public void test_conversion_of_java_doubles_to_little_endian_bytes() {
        toBytes(Converter.of(doubles, ByteOrder.LITTLE_ENDIAN),
                doubles,
                littleEndianBytes);
    }

    public void test_conversion_of_java_doubles_is_big_endian_by_default() {
        toBytes(Converter.of(doubles),
                doubles,
                bigEndianBytes);
    }


    // Converting from bytes

    private void fromBytes(Converter converter, byte[] bytes, double[] expected) {
        ByteBuffer buf = ByteBuffer.wrap(bytes);
        double[] actual = (double[]) converter.convertToJavaFormat(buf, expected.length);
        assertTrue(Arrays.equals(expected, actual));
    }

    public void test_conversion_of_a_big_endian_raw_buffer_to_doubles() {
        fromBytes(Converter.of(Like.floating(8, ByteOrder.BIG_ENDIAN)),
                  bigEndianBytes,
                  doubles);
    }

    public void test_conversion_of_a_little_endian_raw_buffer_to_doubles() {
        fromBytes(Converter.of(Like.floating(8, ByteOrder.LITTLE_ENDIAN)),
                  littleEndianBytes,
                  doubles);
    }

    public void test_conversion_of_a_raw_buffer_to_doubles_uses_big_endian_by_default() {
        fromBytes(Converter.of(Like.floating(8)),
                  bigEndianBytes,
                  doubles);
    }

}
