/*
 * Copyright (c) 2007 by Brian Marick.
 * Inspired by the Java version of the Hierarchical Data format, from the
 * University of Illinois NCSA and The HDF Group (http://www.hdfgroup.org/).
 * This code is intended for use only for training in test-driven design.
 * See LICENSE.txt
 */

package exampler.hdf.internal.converters;

import junit.framework.TestCase;
import exampler.hdf.internal.Like;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;

public class IntArray_3Byte_ConverterTest extends TestCase {


    public void test_equality() {
        int[] same = { 0xF30201, 3, 5 };
        int[] also = { 0xF30201, 3, 5 };
        int[] diff = { 0xAAAAAA, 5, 5 };

        Converter converter = Converter.of(Like.integer(3));
        assertTrue(converter.equalData(same, also));
        assertFalse(converter.equalData(same, diff));
    }

    private final int[] ints = { 0x00FF7703, 2 };
    private final byte[] bigEndianBytes = { (byte) 0xFF, (byte) 0x77, 3, 0, 0, 2 };
    private final byte[] littleEndianBytes = { 3, (byte) 0x77, (byte) 0xFF, 2, 0, 0 };


    private void toBytes(Converter converter, int[] source, byte[] expected) {
        byte[] destination = {  5, 5, 5, 5, 5, 5 };
        ByteBuffer buf = ByteBuffer.wrap(destination);
        converter.overwriteAsBytes(source, buf);

        assertTrue(Arrays.equals(expected, destination));
        
    }

    public void test_conversion_of_java_ints_to_big_endian_bytes() {
        toBytes(Converter.of(Like.integer(3, ByteOrder.BIG_ENDIAN)),
                ints,
                bigEndianBytes);
    }

    public void test_conversion_of_java_ints_to_little_endian_bytes() {
        toBytes(Converter.of(Like.integer(3, ByteOrder.LITTLE_ENDIAN)),
                ints,
                littleEndianBytes);
    }

    public void test_conversion_of_java_ints_is_big_endian_by_default() {
        toBytes(Converter.of(Like.integer(3)),
                ints,
                bigEndianBytes);
    }

    // From bytes

    private void fromBytes(Converter converter, byte[] bytes, int[] expected) {
        ByteBuffer buf = ByteBuffer.wrap(bytes);
        int[] actual = (int[]) converter.convertToJavaFormat(buf, expected.length);
        assertTrue(Arrays.equals(expected, actual));
    }

    public void test_conversion_of_a_big_endian_raw_buffer_to_ints() {
        fromBytes(Converter.of(Like.integer(3, ByteOrder.BIG_ENDIAN)),
                  bigEndianBytes, ints);
    }


    public void test_conversion_of_a_little_endian_raw_buffer_to_ints() {
        fromBytes(Converter.of(Like.integer(3, ByteOrder.LITTLE_ENDIAN)),
                  littleEndianBytes, ints);
    }

    public void test_conversion_of_a_raw_buffer_to_ints_uses_big_endian_by_default() {
        fromBytes(Converter.of(Like.integer(3)),
                  bigEndianBytes, ints);
    }


    // Have to be careful about the way Java sign-extends bytes practically if
    // you just look at them.

    public void test_avoidance_of_sign_extension() {
        fromBytes(Converter.of(Like.integer(3, ByteOrder.BIG_ENDIAN)),
                new byte[] { (byte) 0xFF, (byte) 0xFF, (byte) 0xFF },
                new int[] { 0x00FFFFFF } );
    }

}
