/*
 * Copyright (c) 2007 by Brian Marick.
 * Inspired by the Java version of the Hierarchical Data format, from the
 * University of Illinois NCSA and The HDF Group (http://www.hdfgroup.org/).
 * This code is intended for use only for training in test-driven design.
 * See LICENSE.txt
 */

package exampler.hdf.internal.converters;

import junit.framework.TestCase;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;

public class IntArray_4Byte_ConverterTest extends TestCase {

    // equality
    public void test_can_compare_arrays_of_ints() {
        Converter defaultConverter = Converter.of(new int[3]);
        int[] one = {1, 2, 3 };
        int[] same = {1, 2, 3 };
        int[] diff = {10, 2, 3 };
        assertTrue(defaultConverter.equalData(one, same));
        assertFalse(defaultConverter.equalData(one, diff));
    }


    // converting to bytes
    private final int[] ints = new int[]{ -1, 0x03fffaf0 };
    private final byte[] bigEndianBytes =    { (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF,
                                                  3, (byte)0xFF, (byte)0xFA, (byte)0xF0 };
    @SuppressWarnings({"OctalInteger"})
    private final byte[] littleEndianBytes = { (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF,
                                         (byte)0xF0, (byte)0xFA, (byte)0xFF,         03  };

    
    private void toBytes(Converter converter, int[] source, byte[] expected) {

        byte[] destination = {  5, 5, 5, 5, 5, 5, 5, 5 };
        ByteBuffer buf = ByteBuffer.wrap(destination);
        converter.overwriteAsBytes(source, buf);

        assertTrue(Arrays.equals(expected, destination));
    }

    public void test_conversion_of_java_ints_to_big_endian_bytes() {
        toBytes(Converter.of(ints, ByteOrder.BIG_ENDIAN), ints, bigEndianBytes);
    }

    public void test_conversion_of_java_ints_to_little_endian_bytes() {
        toBytes(Converter.of(ints, ByteOrder.LITTLE_ENDIAN), ints, littleEndianBytes);
    }

    public void test_conversion_of_java_ints_uses_big_endian_as_default() {
        toBytes(Converter.of(ints), ints, bigEndianBytes);
    }




    // Converting from bytes

    private void fromBytes(Converter converter, byte[] source, int[] expected) {
        ByteBuffer buf = ByteBuffer.wrap(source);
        assertTrue(Arrays.equals(expected, (int[]) converter.convertToJavaFormat(buf, expected.length)));
    }

    public void test_conversion_of_a_big_endian_raw_buffer_to_ints() {
        fromBytes(Converter.of(new int[0], ByteOrder.BIG_ENDIAN),
                  bigEndianBytes,
                  ints);
    }

    public void test_conversion_of_a_little_endian_raw_buffer_to_ints() {
        fromBytes(Converter.of(new int[0], ByteOrder.LITTLE_ENDIAN),
                  littleEndianBytes,
                  ints);
    }

    public void test_conversion_of_a_raw_buffer_to_ints_uses_big_endian_by_default() {
        fromBytes(Converter.of(new int[0]),
                  bigEndianBytes,
                  ints);
    }


}
