/*
 * Copyright (c) 2007 by Brian Marick.
 * Inspired by the Java version of the Hierarchical Data format, from the
 * University of Illinois NCSA and The HDF Group (http://www.hdfgroup.org/).
 * This code is intended for use only for training in test-driven design.
 * See LICENSE.txt
 */

package exampler.hdf.internal.converters;

import junit.framework.TestCase;
import exampler.hdf.internal.Like;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;

public class LongArray_8Byte_ConverterTest extends TestCase {
    private final long[] longs = new long[] { -1, 0x03A004B005C006D0l };
    private final byte[] bigEndianBytes = new byte[]{
            (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF,
            (byte)0x03, (byte)0xA0, (byte)0x04, (byte)0xB0, (byte)0x05, (byte)0xC0, (byte)0x06, (byte)0xD0 };
    private final byte[] littleEndianBytes = new byte[]{
            (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF, (byte)0xFF,
            (byte)0xD0, (byte)0x06, (byte)0xC0, (byte)0x05, (byte)0xB0, (byte)0x04, (byte)0xA0, (byte)0x03 };

    // equality

    public void test_can_compare_arrays_of_longs() {
        Converter defaultConverter = Converter.of(new long[3]);
        long[] one = {1l, 2l, 3l };
        long[] same = {1l, 2l, 3l };
        long[] diff = {10l, 2l, 3l };
        assertTrue(defaultConverter.equalData(one, same));
        assertFalse(defaultConverter.equalData(one, diff));
    }


    // converting to bytes


    private void toBytes(Converter converter, long[] source, byte[] expected) {

        byte[] destination = {  5, 5, 5, 5, 5, 5, 5, 5, 4, 4, 4, 4, 4, 4, 4, 4 };
        ByteBuffer buf = ByteBuffer.wrap(destination);
        converter.overwriteAsBytes(source, buf);

        assertTrue(Arrays.equals(expected, destination));
    }

    public void test_conversion_of_java_longs_to_big_endian_bytes() {
        toBytes(Converter.of(longs, ByteOrder.BIG_ENDIAN), longs, bigEndianBytes);
    }

    public void test_conversion_of_java_longs_to_little_endian_bytes() {
        toBytes(Converter.of(longs, ByteOrder.LITTLE_ENDIAN), longs, littleEndianBytes);
    }

    public void test_conversion_of_java_longs_is_big_endian_by_default() {
        toBytes(Converter.of(longs), longs, bigEndianBytes);
    }



    // From bytes

    private void fromBytes(Converter converter, byte[] source, long[] expected) {
        ByteBuffer buf = ByteBuffer.wrap(source);
        assertTrue(Arrays.equals(expected, (long[]) converter.convertToJavaFormat(buf, expected.length)));
    }

    public void test_conversion_of_a_big_endian_raw_buffer_to_longs() {
        fromBytes(Converter.of(Like.integer(8, ByteOrder.BIG_ENDIAN)),
                  bigEndianBytes,
                  longs);
    }

    public void test_conversion_of_a_little_endian_raw_buffer_to_longs() {
        fromBytes(Converter.of(Like.integer(8, ByteOrder.LITTLE_ENDIAN)),
                  littleEndianBytes,
                  longs);
    }

    public void test_conversion_of_a_raw_buffer_uses_big_endian_by_default() {
        fromBytes(Converter.of(Like.integer(8)),
                  bigEndianBytes,
                  longs);
    }

}
