/*
 * Copyright (c) 2007 by Brian Marick.
 * See LICENSE.txt
 */

package exampler.hdf.internal.diskAccess;

import exampler.hdf.CompoundDataset;
import exampler.hdf.internal.CompoundDatasetFAQ;
import exampler.hdf.internal.Errors;
import exampler.hdf.testUtilities.TestFileUtils;

import java.io.RandomAccessFile;
import java.nio.ByteOrder;

// The PileLayout isolates all knowledge about metadata on disk. It is thus an
// answer of frequently asked questions.

public class PileLayoutTest extends TestFileUtils implements CompoundDatasetFAQ {
    private PileLayout layout;

    public int tellHowManyMembers() { return 3; }
    public long tellMemberDimensions() { return 32; }

    public String tellTypeOf(int member) {
        switch(member) {
            case 0: return CompoundDataset.FLOAT_TYPE;
            case 1: return CompoundDataset.FLOAT_TYPE;
            case 2: return CompoundDataset.INTEGER_TYPE;
        }
        Errors.panic("Bad member", member);
        return null;
    }

    public int tellSizeInBytesOf(int member) {
        switch(member) {
            case 0: return 4;
            case 1: return 8;
            case 2: return 8;
        }
        Errors.panic("Bad member", member);
        return 0;
    }

    public ByteOrder tellByteOrderOf(int member) {
        switch(member) {
            case 0: return ByteOrder.BIG_ENDIAN;
            case 1: return ByteOrder.LITTLE_ENDIAN;
            case 2: return ByteOrder.BIG_ENDIAN;
        }
        Errors.panic("Bad member", member);
        return null;
    }




    protected void setUp() {
        super.setUp();
        layout = PileLayout.createFrom(this);
    }


    public void test_creation_is_by_extracting_information_from_a_FAQ() {
        assertEqualContents(this, layout);
    }


    public void test_metadata_knows_how_much_room_is_required_for_data() {

        assertEquals(PileLayout.DIMENSION_SIZE + PileLayout.NUMBER_OF_MEMBERS_SIZE +
                        PileLayout.MEMBER_TYPE_ELEMENT_SIZE * 3 +
                        PileLayout.MEMBER_SIZE_ELEMENT_SIZE * 3 +
                        PileLayout.MEMBER_BYTE_ORDER_ELEMENT_SIZE * 3,
                     layout.tellHowManyBytesOfMetadata());

        assertEquals(layout.tellHowManyBytesOfMetadata() +
                        4 * 32 +   // room for floats
                        8 * 32 +   // room for doubles
                        8 * 32,    // room for longs,
                     layout.tellHowManyBytesDataSetRequires());

    }

    // TODO: this doesn't test exact disk layout.
    // It should, were this to be interoperable with some other app.
    public void test_image_on_disk() throws Exception {
        RandomAccessFile stream = new RandomAccessFile(tempFileName, "rw");
        layout.write(stream);
        stream.close();

        RandomAccessFile newStream = new RandomAccessFile(tempFileName, "r");
        PileLayout newLayout = PileLayout.createFrom(newStream);
        stream.close();

        assertEqualContents(layout, newLayout);
    }


    private void assertEqualContents(CompoundDatasetFAQ one, CompoundDatasetFAQ other) {
        assertEquals(one.tellMemberDimensions(), other.tellMemberDimensions());
        assertEquals(one.tellHowManyMembers(), other.tellHowManyMembers());

        assertEquals(one.tellTypeOf(0), other.tellTypeOf(0));
        assertEquals(one.tellSizeInBytesOf(0), other.tellSizeInBytesOf(0));
        assertEquals(one.tellByteOrderOf(0), other.tellByteOrderOf(0));

        assertEquals(one.tellTypeOf(1), other.tellTypeOf(1));
        assertEquals(one.tellSizeInBytesOf(1), other.tellSizeInBytesOf(1));
        assertEquals(one.tellByteOrderOf(1), other.tellByteOrderOf(1));

        assertEquals(one.tellTypeOf(2), other.tellTypeOf(2));
        assertEquals(one.tellSizeInBytesOf(2), other.tellSizeInBytesOf(2));
        assertEquals(one.tellByteOrderOf(2), other.tellByteOrderOf(2));
    }
}
